from pyscanning.wrapper import BaseScanner, BaseScannerCollection
import Image
import os

class ScannerCollection(BaseScannerCollection):
    def __init__(self):
        self.__devices__ = []
 
    def __refresh__(self):
        self.__devices__ = []
        scanner = Scanner(0, "Pyscan", "Test Device")
        self.__devices__.append(scanner)
    
    def get(self, scanner_id):
        self.__refresh__()
        id = int(scanner_id)
        for dev in self.__devices__:
            if dev.id == id:
                return dev
        return None

    def list(self):
        self.__refresh__()
        return tuple([scanner.info() for scanner in self.__devices__])

class Scanner(BaseScanner):  
    def __init__(self, id, manufacturer, name):
        self.id = id
        self.manufacturer = manufacturer
        self.name = name

    def __str__(self):
        return '%s- %s: %s' % (self.id, self.manufacturer, self.name)
    
    def scan(self):
        imgpath = os.path.dirname(__file__) + os.sep + ".." + os.sep + "testdata" + os.sep + "img1.tiff"
        return Image.open(imgpath)
    
    def info(self):
        return {'id': self.id, 
                'manufacturer': self.manufacturer, 
                'name': self.name}
   
    def __close__(self):
        pass



import os

is_test = os.getenv('PYSCANNING_TEST')
if is_test and is_test.lower() == 'true':
    from backends._test_ import ScannerCollection
elif os.name == 'posix':
    from backends._sane_ import ScannerCollection
elif os.name == 'nt':
    from backends._twain_ import ScannerCollection
else:
    #TODO: Log error
    raise Exception('OS not supported')

__scanners__ = ScannerCollection()
del ScannerCollection


def list():
    return __scanners__.list()

def get(id):
    return __scanners__.get(id)



class BaseScannerCollection(object):

    def __init__(self):
        raise NotImplementedError

    def getScanner(self):
        raise NotImplementedError

    def list(self):
        return self

class BaseScanner(object):

    def __init__(self):
        raise NotImplementedError

    def scan(self):
        raise NotImplementedError

    def info(self):
        raise NotImplementedError

    def status(self):
        raise NotImplementedError




from pyscanning.wrapper import BaseScanner, BaseScannerCollection
import sane

class ScannerCollection(BaseScannerCollection):
    def __init__(self):
        self.__devices__ = []
 
    def __refresh__(self):
        for scanner in self.__devices__:
            scanner.__close__()
        self.__devices__ = []
        sane.exit()


        sane.init()
        devices = sane.get_devices()    
        for dev in devices: 
            id = len(self.__devices__)
            try:
                scanner = Scanner(id, dev[0], dev[1], dev[2], dev[3])
                self.__devices__.append(scanner)
            except:
                pass
    
    def get(self, scanner_id):
        self.__refresh__()
        id = int(scanner_id)
        for dev in self.__devices__:
            if dev.id == id:
                return dev
        return None

    def list(self):
        self.__refresh__()
        return tuple([scanner.info() for scanner in self.__devices__])

class Scanner(BaseScanner):  
    def __init__(self, id, device, manufacturer, name, description):
        self.id = id
        self.manufacturer = manufacturer
        self.name = name
        self.description = description
        self.__scanner__ = sane.open(device)

    def __str__(self):
        return '%s- %s: %s' % (self.id, self.manufacturer, self.name)
    
     
    
    def scan(self):
        return self.__scanner__.scan()
    
    def info(self):
        return {'id': self.id, 
                'manufacturer': self.manufacturer, 
                'name': self.name, 
                'description': self.description}
   
    def __close__(self):
        self.__scanner__.close()
    # TODO:     
    #def status(self):
    #    pass



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='pyscanning',
      version='0.2',
      description='A scanner wrapper for sane and twain.',
      author='Sergio Campos',
      author_email='seocam@seocam.net',
      url='http://code.google.com/p/pyscanning',
      packages=find_packages(),
      package_data={'': ['*.tiff']},
      install_requires=[],
     )

